﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EPiServer.Personalization.VisitorGroups;
using System.Web;
using System.Web.Security;
using System.Web.Profile;
using System.Web.Caching;
using EPiServer.Personalization;
using System.Security.Principal;

namespace CriteriaPack.WeatherCriterion
{
    [VisitorGroupCriterion(
        Category = "Visitor",
        Description = "Match the weather in the visitor's address",
        DisplayName = "Weather Criterion",
        LanguagePath="/shall/cms/visitorgroups/criteria/weather")]
    public class WeatherCriterion : CriterionBase<WeatherCriterionModel>
    {
        
        public override bool IsMatch(System.Security.Principal.IPrincipal principal, System.Web.HttpContextBase httpContext)
        {
            Weather weather;
            try
            {
                if (httpContext.User == null)
                    return false;

                object oPostcode = httpContext.Profile.GetPropertyValue("ZipCode");
                if (oPostcode == null)
                    return false;

                string cacheKey = httpContext.User.Identity.Name + "-weather";
                string postcode = oPostcode.ToString();
                
                if (HttpContext.Current.Cache[cacheKey] == null)
                {
                    HttpContext.Current.Cache.Add(cacheKey, WeatherUtility.GetCurrentWeather(postcode), null, DateTime.Now.AddHours(1), Cache.NoSlidingExpiration, CacheItemPriority.Normal, null);
                }

                weather = HttpContext.Current.Cache[cacheKey] as Weather;
            }
            catch { return false;}
            
            return String.IsNullOrEmpty(base.Model.Weather) ? true : StringMatchHelper.IsMatch(weather.Condition.ToLower(), base.Model.Weather.ToLower(), base.Model.WeatherMatchType);
        }

        public override void Subscribe(ICriterionEvents criterionEvents)
        {
            base.Subscribe(criterionEvents);
            criterionEvents.StartRequest += criterionEvents_StartRequest;
        }

        private void criterionEvents_StartRequest(object sender, CriterionEventArgs e)
        {
           
             //you can have your own logic before the IsMatch method is called
        }

        public override void Unsubscribe(ICriterionEvents criterionEvents)
        {
            criterionEvents.StartRequest -= criterionEvents_StartRequest;
            base.Unsubscribe(criterionEvents);
        }
    }


}
